# rxscrite/__init__.py

"""
RxScrite Programming Language
-----------------------------

An educational programming language built with Python.
"""

# You can choose to expose key components here if you want them to be
# accessible directly from `import rxscrite` or `from rxscrite import ...`
# For example:
# from .lexer import Lexer, LexerError
# from .parser import Parser, ParserError
# from .interpreter import Interpreter, RuntimeError
# from .tokens import Token

# For now, we'll keep it simple. The CLI will use relative imports.

VERSION = "0.1.0"